#pragma once

#include <atlfile.h>

namespace httpbase
{

class IWriteBuffer
{
public:
	virtual ~IWriteBuffer() {};
	virtual BOOL Open() = 0;
	virtual int Write( const BYTE *data, int size ) = 0;
	virtual BOOL Close() = 0;
};

class WriteFileBuffer : public IWriteBuffer
{
public:
	WriteFileBuffer(LPCTSTR filename);
	virtual ~WriteFileBuffer();

	BOOL Open();
	int Write( const BYTE *data, int size );
	BOOL Close();

private:
	BOOL				m_open;
	CAtlTemporaryFile	m_file;
	TCHAR				m_filename[_MAX_PATH];
};

class WriteAutoRaisedMemoryBuffer : public IWriteBuffer
{
public:
	WriteAutoRaisedMemoryBuffer( BYTE **output_buffer );
	virtual ~WriteAutoRaisedMemoryBuffer();

	BOOL Open();
	int Write( const BYTE *data, int size );
	BOOL Close();
private:
	BOOL RaiseMemory( const int wantSize );

private:
	BYTE **m_buffer;
	BYTE *m_internalBuffer;
	int m_maxSize;
	int m_currentSize;
};

class WritePreparedMemoryBuffer : public IWriteBuffer
{
public:
	WritePreparedMemoryBuffer( DWORD filesize, BYTE **output_buffer );
	virtual ~WritePreparedMemoryBuffer();

	BOOL Open();
	int Write( const BYTE *data, int size );
	BOOL Close();

private:
	BYTE *m_internalBuffer;
	BYTE **m_buffer;
	DWORD m_filesize;
	int m_currentSize;
};

};